@@ -3,7 +3,22 @@
 // ArthurOutputDev.cc
 //
 // Copyright 2003 Glyph & Cog, LLC
-// Copyright 2004 Red Hat, Inc
+//
+//========================================================================
+
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
+// Copyright (C) 2005-2008 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008 Pino Toscano <pino@kde.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
 //
 //========================================================================
 
@@ -71,27 +86,26 @@ ArthurOutputDev::ArthurOutputDev(QPainter *painter):
 {
   m_currentBrush = QBrush(Qt::SolidPattern);
   m_fontEngine = 0;
+  m_font = 0;
+  m_image = 0;
 }
 
 ArthurOutputDev::~ArthurOutputDev()
 {
+  delete m_fontEngine;
 }
 
 void ArthurOutputDev::startDoc(XRef *xrefA) {
-  int i;
-
   xref = xrefA;
-  if (m_fontEngine) {
-    delete m_fontEngine;
-  }
+  delete m_fontEngine;
   m_fontEngine = new SplashFontEngine(
 #if HAVE_T1LIB_H
-				    globalParams->getEnableT1lib(),
+  globalParams->getEnableT1lib(),
 #endif
 #if HAVE_FREETYPE_FREETYPE_H || HAVE_FREETYPE_H
-				    globalParams->getEnableFreeType(),
+  globalParams->getEnableFreeType(),
 #endif
-				    globalParams->getAntialias());
+  m_painter->testRenderHint(QPainter::TextAntialiasing));
 }
 
 void ArthurOutputDev::startPage(int pageNum, GfxState *state)
@@ -127,16 +141,7 @@ void ArthurOutputDev::restoreState(GfxState *state)
 
 void ArthurOutputDev::updateAll(GfxState *state)
 {
-  updateLineDash(state);
-  updateLineJoin(state);
-  updateLineCap(state);
-  updateLineWidth(state);
-  updateFlatness(state);
-  updateMiterLimit(state);
-  updateFillColor(state);
-  updateStrokeColor(state);
-  updateFillOpacity(state);
-  updateStrokeOpacity(state);
+  OutputDev::updateAll(state);
   m_needFontUpdate = gTrue;
 }
 
@@ -210,7 +215,7 @@ void ArthurOutputDev::updateFillColor(GfxState *state)
   GfxRGB rgb;
   QColor brushColour = m_currentBrush.color();
   state->getFillRGB(&rgb);
-  brushColour.setRgbF(rgb.r, rgb.g, rgb.b, brushColour.alphaF());
+  brushColour.setRgbF(colToDbl(rgb.r), colToDbl(rgb.g), colToDbl(rgb.b), brushColour.alphaF());
   m_currentBrush.setColor(brushColour);
 }
 
@@ -219,7 +224,7 @@ void ArthurOutputDev::updateStrokeColor(GfxState *state)
   GfxRGB rgb;
   QColor penColour = m_currentPen.color();
   state->getStrokeRGB(&rgb);
-  penColour.setRgbF(rgb.r, rgb.g, rgb.b, penColour.alphaF());
+  penColour.setRgbF(colToDbl(rgb.r), colToDbl(rgb.g), colToDbl(rgb.b), penColour.alphaF());
   m_currentPen.setColor(penColour);
   m_painter->setPen(m_currentPen);
 }
@@ -249,15 +254,16 @@ void ArthurOutputDev::updateFont(GfxState *state)
   FoFiTrueType *ff;
   Ref embRef;
   Object refObj, strObj;
-  GooString *fileName, *substName;
+  GooString *fileName;
   char *tmpBuf;
   int tmpBufLen;
   Gushort *codeToGID;
   DisplayFontParam *dfp;
-  double m11, m12, m21, m22, w1, w2;
+  double *textMat;
+  double m11, m12, m21, m22, fontSize;
   SplashCoord mat[4];
-  char *name;
-  int c, substIdx, n, code;
+  int substIdx, n;
+  int faceIndex = 0;
 
   m_needFontUpdate = false;
   m_font = NULL;
@@ -307,6 +313,7 @@ void ArthurOutputDev::updateFont(GfxState *state)
       case displayFontTT:
 	fileName = dfp->tt.fileName;
 	fontType = gfxFont->isCIDFont() ? fontCIDType2 : fontTrueType;
+	faceIndex = dfp->tt.faceIndex;
 	break;
       }
     }
@@ -341,16 +348,35 @@ void ArthurOutputDev::updateFont(GfxState *state)
 	goto err2;
       }
       break;
-    case fontTrueType:
-      if (!(ff = FoFiTrueType::load(fileName->getCString()))) {
+    case fontType1COT:
+      if (!(fontFile = m_fontEngine->loadOpenTypeT1CFont(
+			   id,
+			   fontsrc,
+			   ((Gfx8BitFont *)gfxFont)->getEncoding()))) {
+	error(-1, "Couldn't create a font for '%s'",
+	      gfxFont->getName() ? gfxFont->getName()->getCString()
+	                         : "(unnamed)");
 	goto err2;
       }
-      codeToGID = ((Gfx8BitFont *)gfxFont)->getCodeToGIDMap(ff);
-      delete ff;
+      break;
+    case fontTrueType:
+    case fontTrueTypeOT:
+	if (fileName)
+	 ff = FoFiTrueType::load(fileName->getCString());
+	else
+	ff = FoFiTrueType::make(tmpBuf, tmpBufLen);
+      if (ff) {
+	codeToGID = ((Gfx8BitFont *)gfxFont)->getCodeToGIDMap(ff);
+	n = 256;
+	delete ff;
+      } else {
+	codeToGID = NULL;
+	n = 0;
+      }
       if (!(fontFile = m_fontEngine->loadTrueTypeFont(
 			   id,
 			   fontsrc,
-			   codeToGID, 256))) {
+			   codeToGID, n))) {
 	error(-1, "Couldn't create a font for '%s'",
 	      gfxFont->getName() ? gfxFont->getName()->getCString()
 	                         : "(unnamed)");
@@ -368,15 +394,41 @@ void ArthurOutputDev::updateFont(GfxState *state)
 	goto err2;
       }
       break;
+    case fontCIDType0COT:
+      if (!(fontFile = m_fontEngine->loadOpenTypeCFFFont(
+			   id,
+			   fontsrc))) {
+	error(-1, "Couldn't create a font for '%s'",
+	      gfxFont->getName() ? gfxFont->getName()->getCString()
+	                         : "(unnamed)");
+	goto err2;
+      }
+      break;
     case fontCIDType2:
-      n = ((GfxCIDFont *)gfxFont)->getCIDToGIDLen();
-      codeToGID = (Gushort *)gmallocn(n, sizeof(Gushort));
-      memcpy(codeToGID, ((GfxCIDFont *)gfxFont)->getCIDToGID(),
-	     n * sizeof(Gushort));
+    case fontCIDType2OT:
+      codeToGID = NULL;
+      n = 0;
+      if (((GfxCIDFont *)gfxFont)->getCIDToGID()) {
+	n = ((GfxCIDFont *)gfxFont)->getCIDToGIDLen();
+	if (n) {
+	  codeToGID = (Gushort *)gmallocn(n, sizeof(Gushort));
+	  memcpy(codeToGID, ((GfxCIDFont *)gfxFont)->getCIDToGID(),
+		  n * sizeof(Gushort));
+	}
+      } else {
+	if (fileName)
+	  ff = FoFiTrueType::load(fileName->getCString());
+	else
+	  ff = FoFiTrueType::make(tmpBuf, tmpBufLen);
+	if (! ff)
+	  goto err2;
+	codeToGID = ((GfxCIDFont *)gfxFont)->getCodeToGIDMap(ff, &n);
+	delete ff;
+      }
       if (!(fontFile = m_fontEngine->loadTrueTypeFont(
 			   id,
 			   fontsrc,
-			   codeToGID, n))) {
+			   codeToGID, n, faceIndex))) {
 	error(-1, "Couldn't create a font for '%s'",
 	      gfxFont->getName() ? gfxFont->getName()->getCString()
 	                         : "(unnamed)");
@@ -390,14 +442,28 @@ void ArthurOutputDev::updateFont(GfxState *state)
   }
 
   // get the font matrix
-  state->getFontTransMat(&m11, &m12, &m21, &m22);
-  m11 *= state->getHorizScaling();
-  m12 *= state->getHorizScaling();
+  textMat = state->getTextMat();
+  fontSize = state->getFontSize();
+  m11 = textMat[0] * fontSize * state->getHorizScaling();
+  m12 = textMat[1] * fontSize * state->getHorizScaling();
+  m21 = textMat[2] * fontSize;
+  m22 = textMat[3] * fontSize;
+
+  SplashCoord matrix[6];
+  {
+  QMatrix painterMatrix = m_painter->worldMatrix();
+  matrix[0] = painterMatrix.m11();
+  matrix[1] = painterMatrix.m12();
+  matrix[2] = painterMatrix.m21();
+  matrix[3] = painterMatrix.m22();
+  matrix[4] = painterMatrix.dx();
+  matrix[5] = painterMatrix.dy();
+  }
 
   // create the scaled font
   mat[0] = m11;  mat[1] = -m12;
   mat[2] = m21;  mat[3] = -m22;
-  m_font = m_fontEngine->getFont(fontFile, mat);
+  m_font = m_fontEngine->getFont(fontFile, mat, matrix);
 
   return;
 
@@ -495,7 +561,6 @@ void ArthurOutputDev::drawChar(GfxState *state, double x, double y,
   // fill
   if (!(render & 1)) {
     int x0, y0, xFrac, yFrac;
-    SplashGlyphBitmap glyph;
 
     x0 = static_cast<int>(floor(x1));
     xFrac = splashFloor((x1 - x0) * splashFontFraction);
@@ -508,15 +573,20 @@ void ArthurOutputDev::drawChar(GfxState *state, double x, double y,
       qPath.setFillRule(Qt::WindingFill);
       for (int i = 0; i < fontPath->length; ++i) {
 	if (fontPath->flags[i] & splashPathFirst) {
-	  qPath.moveTo(x0+fontPath->pts[i].x, y0+fontPath->pts[i].y);
+	  qPath.moveTo(fontPath->pts[i].x+x0, fontPath->pts[i].y+y0);
 	} else if (fontPath->flags[i] & splashPathCurve) {
-	  qPath.quadTo(x0+fontPath->pts[i].x, y0+fontPath->pts[i].y,
-		       x0+fontPath->pts[i+1].x, y0+fontPath->pts[i+1].y);
+	  qPath.quadTo(fontPath->pts[i].x+x0, fontPath->pts[i].y+y0,
+		       fontPath->pts[i+1].x+x0, fontPath->pts[i+1].y+y0);
 	  ++i;
-	} else if (fontPath->flags[i] & splashPathArcCW) {
-	  qDebug() << "Need to implement arc";
-	} else {
-	  qPath.lineTo(x0+fontPath->pts[i].x, y0+fontPath->pts[i].y);
+	}
+#ifdef __GNUC__
+#warning FIX THIS
+#endif
+// 	else if (fontPath->flags[i] & splashPathArcCW) {
+// 	  qDebug() << "Need to implement arc";
+// 	}
+	else {
+	  qPath.lineTo(fontPath->pts[i].x+x0, fontPath->pts[i].y+y0);
 	}
 	if (fontPath->flags[i] & splashPathLast) {
 	  qPath.closeSubpath();
@@ -526,11 +596,11 @@ void ArthurOutputDev::drawChar(GfxState *state, double x, double y,
       GfxRGB rgb;
       QColor brushColour = m_currentBrush.color();
       state->getFillRGB(&rgb);
-      brushColour.setRgbF(rgb.r, rgb.g, rgb.b, state->getFillOpacity());
+      brushColour.setRgbF(colToDbl(rgb.r), colToDbl(rgb.g), colToDbl(rgb.b), state->getFillOpacity());
       m_painter->setBrush(brushColour);
       QColor penColour = m_currentPen.color();
       state->getStrokeRGB(&rgb);
-      penColour.setRgbF(rgb.r, rgb.g, rgb.b, state->getStrokeOpacity());
+      penColour.setRgbF(colToDbl(rgb.r), colToDbl(rgb.g), colToDbl(rgb.b), state->getStrokeOpacity());
       m_painter->setPen(penColour);
       m_painter->drawPath( qPath );
       m_painter->restore();
@@ -676,8 +746,7 @@ void ArthurOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
   int x, y;
   ImageStream *imgStr;
   Guchar *pix;
-  GfxRGB rgb;
-  int alpha, i;
+  int i;
   double *ctm;
   QMatrix matrix;
   int is_identity_transform;
@@ -730,6 +799,7 @@ void ArthurOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 
   if (m_image == NULL || m_image->isNull()) {
     qDebug() << "Null image";
+    delete imgStr;
     return;
   }
   ctm = state->getCTM();
@@ -737,6 +807,8 @@ void ArthurOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 
   m_painter->setMatrix(matrix, true);
   m_painter->drawImage( QPoint(0,0), *m_image );
+  delete m_image;
+  m_image = 0;
   free (buffer);
   delete imgStr;
 
