@@ -6,6 +6,28 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2005 Kristian Høgsberg <krh@redhat.com>
+// Copyright (C) 2005-2007 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005 Jeff Muizelaar <jrmuizel@nit.ca>
+// Copyright (C) 2005 Jonathan Blandford <jrb@redhat.com>
+// Copyright (C) 2005 Marco Pesenti Gritti <mpg@redhat.com>
+// Copyright (C) 2005, 2006, 2008 Brad Hards <bradh@frogmouth.net>
+// Copyright (C) 2006, 2008 Carlos Garcia Campos <carlosgc@gnome.org>
+// Copyright (C) 2007 Julien Rebetez <julienr@svn.gnome.org>
+// Copyright (C) 2008 Pino Toscano <pino@kde.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #include <config.h>
 
 #ifdef USE_GCC_PRAGMAS
@@ -25,6 +47,7 @@
 #include "PageLabelInfo.h"
 #include "Catalog.h"
 #include "Form.h"
+#include "OptionalContent.h"
 
 //------------------------------------------------------------------------
 // Catalog
@@ -33,6 +56,7 @@
 Catalog::Catalog(XRef *xrefA) {
   Object catDict, pagesDict, pagesDictRef;
   Object obj, obj2;
+  Object optContentProps;
   char *alreadyRead;
   int numPages0;
   int i;
@@ -45,6 +69,7 @@ Catalog::Catalog(XRef *xrefA) {
   baseURI = NULL;
   pageLabelInfo = NULL;
   form = NULL;
+  optContent = NULL;
 
   xref->getCatalog(&catDict);
   if (!catDict.isDict()) {
@@ -111,6 +136,9 @@ Catalog::Catalog(XRef *xrefA) {
     obj.dictLookup("EmbeddedFiles", &obj2);
     embeddedFileNameTree.init(xref, &obj2);
     obj2.free();
+    obj.dictLookup("JavaScript", &obj2);
+    jsNameTree.init(xref, &obj2);
+    obj2.free();
   }
   obj.free();
 
@@ -171,6 +199,12 @@ Catalog::Catalog(XRef *xrefA) {
   // get the outline dictionary
   catDict.dictLookup("Outlines", &outline);
 
+  // get the Optional Content dictionary
+  if (catDict.dictLookup("OCProperties", &optContentProps)->isDict()) {
+    optContent = new OCGs(&optContentProps, xref);
+  }
+  optContentProps.free();
+
   // perform form-related loading after all widgets have been loaded
   if (form) 
     form->postWidgetsLoad();
@@ -203,11 +237,13 @@ Catalog::~Catalog() {
   dests.free();
   destNameTree.free();
   embeddedFileNameTree.free();
+  jsNameTree.free();
   if (baseURI) {
     delete baseURI;
   }
   delete pageLabelInfo;
   delete form;
+  delete optContent;
   metadata.free();
   structTreeRoot.free();
   outline.free();
@@ -375,106 +411,63 @@ LinkDest *Catalog::findDest(GooString *name) {
 EmbFile *Catalog::embeddedFile(int i)
 {
     Object efDict;
-    Object fileSpec;
-    Object fileDesc;
-    Object paramDict;
-    Object paramObj;
-    Object strObj;
-    Object obj, obj2;
+    Object obj;
     obj = embeddedFileNameTree.getValue(i);
-    GooString *fileName = new GooString();
-    char *descString = embeddedFileNameTree.getName(i)->getCString();
-    GooString *desc = new GooString(descString);
-    GooString *createDate = new GooString();
-    GooString *modDate = new GooString();
-    GooString *checksum = new GooString();
-    Stream *efStream = NULL;
-    int size = -1;
+    EmbFile *embeddedFile = 0;
     if (obj.isRef()) {
-	if (obj.fetch(xref, &efDict)->isDict()) {
-	    // efDict matches Table 3.40 in the PDF1.6 spec
-	    efDict.dictLookup("F", &fileSpec);
-	    if (fileSpec.isString()) {
-		delete fileName;
-		fileName = new GooString(fileSpec.getString());
-	    }
-	    fileSpec.free();
-
-	    // the logic here is that the description from the name
-	    // dictionary is used if we don't have a more specific
-	    // description - see the Note: on page 157 of the PDF1.6 spec
-	    efDict.dictLookup("Desc", &fileDesc);
-	    if (fileDesc.isString()) {
-		delete desc;
-		desc = new GooString(fileDesc.getString());
-	    } else {
-		efDict.dictLookup("Description", &fileDesc);
-		if (fileDesc.isString()) {
-		    delete desc;
-		    desc = new GooString(fileDesc.getString());
-		}
-	    }
-	    fileDesc.free();
-	    
-	    efDict.dictLookup("EF", &obj2);
-	    if (obj2.isDict()) {
-		// This gives us the raw data stream bytes
-
-		obj2.dictLookup("F", &strObj);
-		if (strObj.isStream()) {
-		    efStream = strObj.getStream();
-		}
-
-		// dataDict corresponds to Table 3.41 in the PDF1.6 spec.
-		Dict *dataDict = efStream->getDict();
-
-		// subtype is normally mimetype. You can extract it with code like this:
-		// Object subtypeName;
-		// dataDict->lookup( "Subtype", &subtypeName );
-		// It is optional, so this will sometimes return a null object
-		// if (subtypeName.isName()) {
-		//        std::cout << "got subtype name: " << subtypeName.getName() << std::endl;
-		// }
-
-		// paramDict corresponds to Table 3.42 in the PDF1.6 spec
-		Object paramDict;
-		dataDict->lookup( "Params", &paramDict );
-		if (paramDict.isDict()) {
-		    paramDict.dictLookup("ModDate", &paramObj);
-		    if (paramObj.isString()) {
-		        delete modDate;
-		        modDate = new GooString(paramObj.getString());
-		    }
-		    paramObj.free();
-		    paramDict.dictLookup("CreationDate", &paramObj);
-		    if (paramObj.isString()) {
-		        delete createDate;
-		        createDate = new GooString(paramObj.getString());
-		    }
-		    paramObj.free();
-		    paramDict.dictLookup("Size", &paramObj);
-		    if (paramObj.isInt()) {
-		        size = paramObj.getInt();
-		    }
-		    paramObj.free();
-		    paramDict.dictLookup("CheckSum", &paramObj);
-		    if (paramObj.isString()) {
-		        delete checksum;
-		        checksum = new GooString(paramObj.getString());
-		    }
-		    paramObj.free();
-		}
-		paramDict.free();
-	    }
-	    efDict.free();
-	    obj2.free();
-	}
+        GooString desc(embeddedFileNameTree.getName(i));
+        embeddedFile = new EmbFile(obj.fetch(xref, &efDict), &desc);
+        efDict.free();
+    } else {
+        Object null;
+        embeddedFile = new EmbFile(&null);
     }
-    EmbFile *embeddedFile = new EmbFile(fileName, desc, size, createDate, modDate, checksum, strObj);
-    strObj.free();
     return embeddedFile;
 }
 
+GooString *Catalog::getJS(int i)
+{
+  Object obj = jsNameTree.getValue(i);
+  if (obj.isRef()) {
+    Ref r = obj.getRef();
+    obj.free();
+    xref->fetch(r.num, r.gen, &obj);
+  }
+
+  if (!obj.isDict()) {
+    obj.free();
+    return 0;
+  }
+  Object obj2;
+  if (!obj.dictLookup("S", &obj2)->isName()) {
+    obj2.free();
+    obj.free();
+    return 0;
+  }
+  if (strcmp(obj2.getName(), "JavaScript")) {
+    obj2.free();
+    obj.free();
+    return 0;
+  }
+  obj.dictLookup("JS", &obj2);
+  GooString *js = 0;
+  if (obj2.isString()) {
+    js = new GooString(obj2.getString());
+  }
+  else if (obj2.isStream()) {
+    Stream *stream = obj2.getStream();
+    js = new GooString();
+    stream->reset();
+    int i;
+    while ((i = stream->getChar()) != EOF) {
+      js->append((char)i);
+    }
+  }
+  obj2.free();
+  obj.free();
+  return js;
+}
+
 NameTree::NameTree()
 {
   size = 0;
@@ -560,7 +553,6 @@ int NameTree::Entry::cmp(const void *voidKey, const void *voidEntry)
 GBool NameTree::lookup(GooString *name, Object *obj)
 {
   Entry **entry;
-  char *cname;
 
   entry = (Entry **) bsearch(name, entries,
 			     length, sizeof(Entry *), Entry::cmp);
@@ -636,3 +628,106 @@ GBool Catalog::indexToLabel(int index, GooString *label)
     return gTrue;
   }
 }
+
+EmbFile::EmbFile(Object *efDict, GooString *description)
+{
+  m_name = 0;
+  m_description = 0;
+  if (description)
+    m_description = description->copy();
+  m_size = -1;
+  m_createDate = 0;
+  m_modDate = 0;
+  m_checksum = 0;
+  m_mimetype = 0;
+  if (efDict->isDict()) {
+    Object fileSpec;
+    Object fileDesc;
+    Object paramDict;
+    Object paramObj;
+    Object obj2;
+    Stream *efStream = NULL;
+    // efDict matches Table 3.40 in the PDF1.6 spec
+    efDict->dictLookup("F", &fileSpec);
+    if (fileSpec.isString()) {
+      m_name = new GooString(fileSpec.getString());
+    }
+    fileSpec.free();
+
+    // the logic here is that the description from the name
+    // dictionary is used if we don't have a more specific
+    // description - see the Note: on page 157 of the PDF1.6 spec
+    efDict->dictLookup("Desc", &fileDesc);
+    if (fileDesc.isString()) {
+      delete m_description;
+      m_description = new GooString(fileDesc.getString());
+    } else {
+      efDict->dictLookup("Description", &fileDesc);
+      if (fileDesc.isString()) {
+        delete m_description;
+        m_description = new GooString(fileDesc.getString());
+      }
+    }
+    fileDesc.free();
+
+    efDict->dictLookup("EF", &obj2);
+    if (obj2.isDict()) {
+      // This gives us the raw data stream bytes
+
+      obj2.dictLookup("F", &m_objStr);
+      if (m_objStr.isStream()) {
+        efStream = m_objStr.getStream();
+
+        // dataDict corresponds to Table 3.41 in the PDF1.6 spec.
+        Dict *dataDict = efStream->getDict();
+
+        // subtype is normally the mimetype
+        Object subtypeName;
+        if (dataDict->lookup("Subtype", &subtypeName)->isName()) {
+          m_mimetype = new GooString(subtypeName.getName());
+        }
+        subtypeName.free();
+
+        // paramDict corresponds to Table 3.42 in the PDF1.6 spec
+        Object paramDict;
+        dataDict->lookup( "Params", &paramDict );
+        if (paramDict.isDict()) {
+          paramDict.dictLookup("ModDate", &paramObj);
+          if (paramObj.isString()) {
+            m_modDate = new GooString(paramObj.getString());
+          }
+          paramObj.free();
+          paramDict.dictLookup("CreationDate", &paramObj);
+          if (paramObj.isString()) {
+            m_createDate = new GooString(paramObj.getString());
+          }
+          paramObj.free();
+          paramDict.dictLookup("Size", &paramObj);
+          if (paramObj.isInt()) {
+            m_size = paramObj.getInt();
+          }
+          paramObj.free();
+          paramDict.dictLookup("CheckSum", &paramObj);
+          if (paramObj.isString()) {
+            m_checksum = new GooString(paramObj.getString());
+          }
+          paramObj.free();
+        }
+        paramDict.free();
+      }
+    }
+    obj2.free();
+  }
+  if (!m_name)
+    m_name = new GooString();
+  if (!m_description)
+    m_description = new GooString();
+  if (!m_createDate)
+    m_createDate = new GooString();
+  if (!m_modDate)
+    m_modDate = new GooString();
+  if (!m_checksum)
+    m_checksum = new GooString();
+  if (!m_mimetype)
+    m_mimetype = new GooString();
+}
