@@ -1,5 +1,8 @@
 /* poppler-document.cc: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
+ * Copyright (C) 2005-2008, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2006, Stefan Kebekus <stefan.kebekus@math.uni-koeln.de>
+ * Copyright (C) 2006, Wilfried Huss <Wilfried.Huss@gmx.at>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <poppler-qt.h>
@@ -26,10 +29,19 @@
 #include <ErrorCodes.h>
 #include <SplashOutputDev.h>
 #include <splash/SplashBitmap.h>
+#include <DateInfo.h>
 #include "poppler-private.h"
 
 namespace Poppler {
 
+static GooString *QStringToGooString(const QString &s) {
+    int len = s.length();
+    char *cstring = (char *)gmallocn(s.length(), sizeof(char));
+    for (int i = 0; i < len; ++i)
+      cstring[i] = s.at(i).unicode();
+    return new GooString(cstring, len);
+}
+
 Document *Document::load(const QString &filePath)
 {
   if (!globalParams) {
@@ -70,8 +82,9 @@ bool Document::unlock(const QCString &password)
 {
   if (data->locked) {
     /* racier then it needs to be */
+    GooString *filename = new GooString(data->doc.getFileName());
     GooString *pwd = new GooString(password.data());
-    DocumentData *doc2 = new DocumentData(data->doc.getFileName(), pwd);
+    DocumentData *doc2 = new DocumentData(filename, pwd);
     delete pwd;
     if (!doc2->doc.isOk()) {
       delete doc2;
@@ -79,6 +92,7 @@ bool Document::unlock(const QCString &password)
       delete data;
       data = doc2;
       data->locked = false;
+      data->m_fontInfoScanner = new FontInfoScanner(&(data->doc));
     }
   }
   return data->locked;
@@ -132,6 +146,7 @@ bool Document::scanForFonts( int numPages, QValueList<FontInfo> *fontList ) cons
                   (Poppler::FontInfo::Type)((::FontInfo*)items->get(i))->getType());
     fontList->append(font);
   }
+  deleteGooList(items, ::FontInfo);
   return true;
 }
 
@@ -206,35 +221,17 @@ QDateTime Document::getDate( const QString & type ) const
   }
 
   Object obj;
-  char *s;
-  int year, mon, day, hour, min, sec;
+  int year, mon, day, hour, min, sec, tz_hour, tz_minute;
+  char tz;
   Dict *infoDict = info.getDict();
   QString result;
 
   if ( infoDict->lookup( (char*)type.latin1(), &obj )->isString() )
   {
     QString s = UnicodeParsedString(obj.getString());
-    if ( s[0] == 'D' && s[1] == ':' )
-      s = s.mid(2);
-
-    /* FIXME process time zone on systems that support it */  
-    if ( sscanf( s.latin1(), "%4d%2d%2d%2d%2d%2d", &year, &mon, &day, &hour, &min, &sec ) == 6 )
+    // TODO do something with the timezone information
+    if ( parseDateString( s.latin1(), &year, &mon, &day, &hour, &min, &sec, &tz, &tz_hour, &tz_minute ) )
     {
-      /* Workaround for y2k bug in Distiller 3 stolen from gpdf, hoping that it won't
-       *   * be used after y2.2k */
-      if ( year < 1930 && strlen (s) > 14) {
-	int century, years_since_1900;
-	if ( sscanf( s, "%2d%3d%2d%2d%2d%2d%2d",
-	    &century, &years_since_1900,
-	    &mon, &day, &hour, &min, &sec) == 7 )
-	  year = century * 100 + years_since_1900;
-	else {
-	  obj.free();
-	  info.free();
-	  return QDateTime();
-	}
-      }
-
       QDate d( year, mon, day );  //CHECK: it was mon-1, Jan->0 (??)
       QTime t( hour, min, sec );
       if ( d.isValid() && t.isValid() ) {
@@ -322,7 +319,7 @@ bool Document::print(const QString &file, QValueList<int> pageList, double hDPI,
   if (psOut->isOk()) {
     QValueList<int>::iterator it;
     for (it = pageList.begin(); it != pageList.end(); ++it )
-      data->doc.displayPage(psOut, *it, hDPI, vDPI, rotate, gFalse, gTrue, gFalse);
+      data->doc.displayPage(psOut, *it, hDPI, vDPI, rotate, gFalse, gTrue, gTrue);
     
     delete psOut;
     return true;
